﻿using System.Collections;
using System.Globalization;

using NUnit.Framework;

using GamesStore.Web.Helpers;

namespace GamesStore.Web.Tests
{
    public class CultureHelperTests
    {
        [TestFixture]
        public class GetSupportedCultureTests
        {
            [Test]
            public void GetSupportedCulture_Null_ReturnsDefaultCulture()
            {
                var actual = CultureHelper.GetSupportedCulture(null);

                var expected = CultureHelper.DEFAULT_CULTURE;

                Assert.AreEqual(expected, actual, "No parameters should fall back to the default culture");
            }

            [Test]
            public void GetSupportedCulture_SupportedCulture_ReturnsTheCulture()
            {
                var actual = CultureHelper.GetSupportedCulture("sk-SK");
                var expected = new CultureInfo("sk-SK");

                Assert.AreEqual(expected, actual, "Should have returned the supported culture.");
            }


            [Test]
            public void GetSupportedCulture_Nonsense_ReturnsDefaultCulture()
            {
                var actual = CultureHelper.GetSupportedCulture("", "asdf");

                Assert.AreEqual(CultureHelper.DEFAULT_CULTURE, actual, "Should have been the default culture.");
            }


            [Test]
            public void GetSupportedCulture_RealYetUnsupportedCulture_ReturnsDefaultCulture()
            {
                var actual = CultureHelper.GetSupportedCulture(new CultureInfo("de-de").Name);

                Assert.AreEqual(CultureHelper.DEFAULT_CULTURE, actual, "Unsupported culture should fallback to the default culture.");
            }


            [Test]
            public void GetSupportedCulture_RealSupportedCulture_ReturnsTheCulture()
            {
                var expected = new CultureInfo("sk-sk");

                var actual = CultureHelper.GetSupportedCulture("sk-sk");

                Assert.AreEqual(expected, actual, "Should have returned the given culture.");
            }


            [TestCase("en-US", "de-DE", "fr-FR")]
            [TestCase("en-US", "en-GB", TestName = "GetSupportedCulture_Cases(\"en-US\", \"en-DB\")")]
            [TestCase("en-US", "en", "sk-SK")]
            public void GetSupportedCulture_Cases(string expectedCulture, params string[] cultureCodes)
            {
                var actual = CultureHelper.GetSupportedCulture(cultureCodes);

                Assert.AreEqual(expectedCulture, actual.Name, "Should have returned closest supported culture.");
            }

            public static IEnumerable CultureCases
            {
                get
                {
                    yield return new TestCaseData(new CultureInfo("en-US"), new[] { "en-GB" });
                    yield return new TestCaseData(new CultureInfo("en-US"), new[] { "de-DE", "cs-CZ" });
                }
            }

            [Test, TestCaseSource("CultureCases")]
            public void GetSupportedCulture_CasesSource(CultureInfo expectedCulture, string[] cultureCodes)
            {
                var actual = CultureHelper.GetSupportedCulture(cultureCodes);

                Assert.AreEqual(expectedCulture, actual, "Should have returned closest supported culture.");
            }
        }
    }
}